#ifndef __GraphicPrim__
#define __GraphicPrim__

#include<libps.h>
#include<stdio.h>

/* This header files describes the most important Graphic Primitives,
like generating a sprite, generating one of the objects and the redrawing
of the screen. All necesarry file IO for the initialization of the texture is done 
here. */

/* The box and the wall object is splitted into different segments. 
This is due to the fact, that wall and box sides should not be painted 
together, if two sides are positioned at the same geometry position. This
causes trouble with the visibility test of the PSX. */

// Returns the Order table for he foreground for the time bar etc.
GsOT *getForeGround();

// Returns ther order table for the background (horizon)
GsOT *getBackGround();

// The midground which contains the 3d scene
GsOT *getMidGround();

// The box ground conatins a box, if the box is moving
GsOT *getBoxGround();



// Initializes the graphics system and the Orderin Tables.
void InitializeGraphic();

// Loads all necesarry geometry and image files as textures
void LoadObjects();


// Reads in an image and generates a sprite structure
GsSPRITE *loadSprite(char *filename);

// Delivers the part of a new box positioned here
GsDOBJ2 getBox(int part,GsCOORDINATE2 *coordinate);

// Delivers a floor element positioned here
GsDOBJ2 getFloor(GsCOORDINATE2 *coordinate);

// Delivers a target element positioned here
GsDOBJ2 getTarget(GsCOORDINATE2 *coordinate);

// Delivers the part of a wall element positioned here
GsDOBJ2 getWall(int part,GsCOORDINATE2 *coordinate);



// Switches the actual frame
void SwitchFrames();


// Shows an image directly
void DisplayStartScreen(char *name);


// Displays the pause screen
void DisplayPause();


// Initializes the WorkSpace ( a lot of malloc)
void InitializeSpaces();

// Shows the main screen
void DisplayMain(int level,int won);


// Sets the fading value (0..255)
void setFadingValue(int param);

// Draws the complete scene during gamelpay including frame switch
void RedrawAll();
#endif